#RT-AC mit HFLI/Stanford
use Lab::Moose;
use Lab::Zhinst;
use Lab::Moose::Stabilizer; #to wait until temperature has converged to the setpoint -> need for T dependent meas
use File::Basename;
use POSIX qw(strftime);
use Time::HiRes qw(sleep);
use Lab::Moose::Countdown;
use 5.010;

my $sample = 'NbN_400_IV_A';
my $start = strftime " %H-%M" , localtime;
print "$start lets start \n";

#------------------------



####Nanovoltmeter Filter Off A.W. 17.10.2022


my $R_pre = 1e6; #1MOhm Vorwiderstand -> 10nA bei 1V (Spannnungsteiler am Femto)

 # my $F_factor = 1e8; #Femto Verstärkungsfaktor
 my $F_factor = 1e7; 
# my $F_factor = 1e4; #Femto Verstärkungsfaktor
 my $divider = 100; #V-divider an Femto

 #my $factor = 1;   #kein Verstärker
#my $factor = 10;  #20dB voltage verstärker
 #my $factor = 1e2; #40dB voltage verstärker
# my $factor = 1e3; #60dBvoltage verstärker 
# my $factor = 1e4; #80dB voltage verstärker

my $lakeshore = instrument(
    type => 'Lakeshore340',
    connection_type => 'VISA_GPIB',
    connection_options => {pad => 12},
    input_channel => 'B', # set default input channel for all method calls
);


my $voltageoutputMag = instrument(				#connection to Yokogawa-voltage source via GPIB
    type => 'YokogawaGS200',
    connection_type => 'VISA_GPIB',
		connection_options => {
		gpib_address => 1,
		timeout => 10,
		
		},
		
		max_units => 200e-3,
		min_units => -200e-3,
		max_units_per_step => 0.1,
		max_units_per_second => 0.1,
		
);

my $voltageoutputCur = instrument(				#connection to Yokogawa-voltage source via GPIB
    type => 'YokogawaGS200',
    connection_type => 'VISA_GPIB',
		connection_options => {
		gpib_address => 5,
		timeout => 10,
		
		},
		
		max_units => 20,
		min_units => -20,
		max_units_per_step => 10,
		max_units_per_second => 10,
		
);

my $DC_MM = instrument(
	type => 'HP34410A',
	  connection_type => 'VISA_GPIB',
	  connection_options => {pad => 22},	
);	


my $nano = instrument(
	type => 'HP34420A',
    connection_type => 'VISA_GPIB',
    connection_options => {pad => 17},
	# input_channel => 1,
);



my $temperaturesweep = sweep( 
    type => 'Step::Temperature',
    instrument => $lakeshore,
    # from => $T_start,
    # to => $T_stop ,
    # step => $T_step ,
	# pid => [200, 200, 1],
	
	 points => [ 4.45,4.6], #NbN
	#list => [4, 4.7, 5], #NbN
	steps => [0.005],
	# steps => [ 0.05, 0.1],
	# points => [$T_start, 2.5], #Alu
	# steps => [ 0.005],
 
    # stabilizer options (details in Lab::Moose::Stabilizer)
	
    #measurement_interval => 1,
    # observation_time => 30,
	tolerance_setpoint => 0.001, #0.002, #0.005, #0.01, #0.0005,#0.001,#0.0001,#0.01, #
    tolerance_std_dev => 0.0015, #0.002, #0.003, #0.005, #0.01, #0.002,#0.0025, #0.0015, #0.01, #
	observation_time => 20, #10, #30, #40, #
);


my $voltagesweep = sweep(
    type       => 'Step::Voltage',
    instrument => $voltageoutputCur,
	delay_before_loop => 0,
	delay_in_loop => .00001,
	 points => [0, -10,10,0], 
	steps => [ -0.25, 0.25,-0.25],
	#from => -10, to => 10, step => 1e-1,
	#backsweep => 1,
   );

# my $magnetsweep = sweep(
    # type       => 'Step::Voltage',
    # instrument => $voltageoutputMag,
	# delay_before_loop => 0,
	# delay_in_loop => .1,
	# from => -200e-3, to => 200e-3, step => 5e-3,
	# #backsweep => 1,
   # );
#--- ------------- save data

my $folder   = datafolder();


my $datafile = sweep_datafile(
	filename => 'R(T)'.$sample,
	columns => ['T' , 'V_in' , 'V_sample' , 'I_in',  'B'] 
);

#$datafile->add_plot(x => 'T', y => 'R', legend => 'B', plot_options => {grid => 1} );
 $datafile->add_plot(x => 'I_in', y => 'V_sample',  refresh_interval => 30, plot_options => {grid => 1});

my $meas = sub {
	my $sweep = shift;
	my $T_mess = $lakeshore->get_T();
	# my $f = $hfli -> get_frequency();
	#my $f = $li -> get_frq();
	# my $B = $ips->get_field();
	my $V_yoko = $voltageoutputMag->cached_level();
	my $B = 0.119*$V_yoko;

# #----HFLI:
	# #1st input: going in
	# my $xy_0 = $hfli->get_xy(demod => 0);
	# #my $Vx = $xy_0->{x}; looks like phase...
	# my $x = $xy_0->{x};
	# my $y = $xy_0->{y};
	# my $V1 = sqrt($x**2 + $y**2);
	#
	# #2nd input: going out
	# my $xy_3 = $hfli->get_xy(demod => 3);
	# #my $Vx = $xy_0->{x}; looks like phase...
	# my $a = $xy_3->{x};
	# my $b = $xy_3->{y};
	# my $V2 = sqrt($a**2 + $b**2);
	#
	# # my $V_in = $amp/100; #due to Femto
	# # my $I = $V1/($F_factor*$R_pre); #current measured going in
	# my $I = $V1/($F_factor ); #current measured going in
	# my $V = $V2 /$factor; #V over sample
	# my $R = $V/$I;
	#
	# $sweep->log( Frequency => $f,  R => $R,  V_in => $V1,  V_sample => $V,  I_in => $I,  T => $T_mess ,  tc => $tc , Vrms => $amp , Vrange => $amp_range);
# #----Stanford:
	# my $xy = $li ->get_xy();
	# my $x = $xy ->{x};
	# my $y = $xy ->{y};
	# my $V = sqrt($x**2 + $y**2);
	
	my $I = $DC_MM->get_value() /  $F_factor; #current going in
	#my $I = $voltageoutputCur->get_level() / $R_pre ; #current going in
	my $V_sample = $nano->get_value(); 
	# my $R = $V_sample / $I;
	
	# $li->set_auto_sens(
		# r => $V,
		# min_sens => 10e-6, # Smallest allowed range of li
		# # min_sens => 1e-6, 
    #);

	# $sweep->log( T => $T_mess, R => $R, V_sample => $V_sample, V_in => $ampl, I_in => $I, Frequency => $f, x=> $x, y=> $y);#, B_offset => $B  );
	$sweep->log( T => $T_mess, V_sample => $V_sample, V_in => $voltageoutputCur->get_level(), I_in => $I, B => $B  );

};

#------------ measurement

# $master_sweep->start(
	# # slave => $slave_sweep,
	# measurement => $meas,
	# datafile => $datafile,
	# # datafile_dim => 1
# );

$temperaturesweep->start(
	slave =>  $voltagesweep,
    measurement => $meas,
    datafile    => $datafile,
    folder      => $folder,
	datafile_dim => 1
);

#---

my $finish = strftime " %H-%M" , localtime;
print "FINISHED! $finish \n";
$lakeshore->set_setpoint(value => 4.4, loop => 1);
# $lakeshore->set_heater_range(value => 0); #Heater off
#$ips->sweep_to_field(target => 0, rate => 0.1); 