use Lab::Moose;
use Lab::Zhinst;
use Lab::Moose::Stabilizer; #to wait until temperature has converged to the setpoint.
use File::Basename;
use POSIX qw(strftime);
use Time::HiRes qw(sleep);
use Lab::Moose::Countdown;
use 5.010;
#------------------------------------------------------------------------------------------include T as slave loop
my $sample = 'NbN_400_IV_A';
my $starttime = strftime " %H-%M" , localtime;
print "$starttime lets start \n";


my $Verstark = 1e2.8; #Miteq Verstärker
my $Daempf = 1e8; #2*20dB+4*10dB

my $delay = 4*60;


my $lakeshore = instrument(
    type => 'Lakeshore340',
    connection_type => 'VISA_GPIB',
    connection_options => {pad => 12},
    input_channel => 'B', # set default input channel for all method calls
);


my $voltageoutputMag = instrument(				#connection to Yokogawa-voltage source via GPIB
    type => 'YokogawaGS200',
    connection_type => 'VISA_GPIB',
		connection_options => {
		gpib_address => 1,
		timeout => 10,
		
		},
		
		max_units => 20e-3,
		min_units => -20e-3,
		max_units_per_step => 0.1,
		max_units_per_second => 0.1,
		
);


my $voltageoutputCur = instrument(				#connection to Yokogawa-voltage source via GPIB
    type => 'YokogawaGS200',
    connection_type => 'VISA_GPIB',
		connection_options => {
		gpib_address => 5,
		timeout => 10,
		
		},
		
		max_units => 20e-3,
		min_units => -20e-3,
		max_units_per_step => 0.1,
		max_units_per_second => 0.1,
		
);



# my $DC_MM = instrument(
	# type => 'HP34410A',
	  # connection_type => 'VISA_GPIB',
	  # connection_options => {pad => 22},	
# );	

my $znl = instrument(
   type => 'RS_ZNL',
   connection_type => 'VISA_GPIB',
   connection_options => {gpib_address => 20 },
   );



my $temperaturesweep = sweep(
	type => 'Step::Temperature',
	instrument => $lakeshore,
	 #from => 4,
	 #to => 4,
	 #step => 1,
	points => [1.3,2,4.475,4.54],
	steps => [ 0.025,2.475,0.001],
	#list => [1],
	tolerance_setpoint => 0.001,#0.0005,#0.0001,#0.01,
    tolerance_std_dev => 0.002,#0.0015, #0.0025, #0.0015, #0.01,
    observation_time => 10,#30,#40,
	delay_in_loop => 30#1*60
);

#my $folder = datafolder();

my $datafile = sweep_datafile(
                        columns => [qw/T magneticFieldVoltage freq Re_S21 Im_S21 amplitude phase/]);




my $meas = sub {
    my $sweep = shift;
	my $T = $lakeshore->get_T();
    my $magneticFieldVoltage = $voltageoutputMag->cached_level();
    my $block = $znl->sparam_sweep(timeout => 5*60);
 
    $sweep->log_block(
        prefix => {T => $T, magneticFieldVoltage => $magneticFieldVoltage},
        block => $block
    );
};
 
 
$lakeshore->set_T(value=>1.3);

countdown(3*60);


 
$temperaturesweep->start(
    measurement => $meas,
    datafile   => $datafile,
	folder => 'L(T)_ZNL_-80dBm_Nachmessung_am_Uebergang',
    #datafile_dim => 1, # each VNA trace in a separate file
    #point_dim => 1, # the measurement sub logs blocks, not points
);



