#R(B) at T>tc
#RT-AC mit HFLI/Stanford
use Lab::Moose;
use Lab::Zhinst;
use Lab::Moose::Stabilizer; #to wait until temperature has converged to the setpoint.
use File::Basename;
use POSIX qw(strftime);
use Time::HiRes qw(sleep);
use Lab::Moose::Countdown;
use 5.010;

my $start = strftime " %H-%M" , localtime;
print "$start lets start \n";

#------------------------
my $sample = 'NbN_400_IV_A';

my $R_pre = 1e9; #10MOhm Vorwiderstand -> 100nA bei 1V

my $factor = 1e2; #20dB voltage verstärker

my $T_set = 6; 
my $T_range = 3;

my $ampl = 5; #1V
my $f_set = 3.333; #3; # Hz


my $B_start = -0.5;
my $B_stop = 9;
# my $B_start = 12;
# my $B_stop = -1;
my $B_rate = 200e-3; 

#-------------------instruments

my $li= instrument( 		#stanford lock-in
    type => 'SR830',
    connection_type => 'VISA_GPIB',
    connection_options => {pad => 15},
);
	
my $lakeshore = instrument(
    type => 'Lakeshore340',
    connection_type => 'VISA_GPIB',
    connection_options => {pad => 12},
    input_channel => 'B', # set default input channel for all method calls
);

my $ips = instrument(
	type => 'OI_IPS',
	connection_type => 'VISA::GPIB',
	connection_options => {pad => 24},
	max_fields => [12],
	max_field_rates => [0.2],
);
 
#-------------------------getting started
$lakeshore->set_heater_range(value => $T_range);
$lakeshore->set_setpoint(value => $T_set, loop => 1);

$li ->set_frq(value => $f_set); 
$li ->set_amplitude(value => $ampl);
# $li ->set_sens(value => 10e-6); #NbN SC #(value => 100e-3); #NbN N
# $li ->set_sens(value => 1e-3);

$ips->sweep_to_field(target => $B_start, rate => 0.2);

# countdown(100);
countdown(300);
my $I = $ampl / $R_pre ; #current going in
my $f = $li -> get_frq();
#----------------------sweep 


#a2
my $b_sweep = sweep(
	type => 'Continuous::Magnet',
	instrument => $ips,
	from => $B_start,
	to => $B_stop,
	rate => $B_rate, # sweep takes  67 min
	interval => 0,
	start_rate => 200e-3,
	delay_before_loop => 10,
	backsweep => 1,
);


my $temperaturesweep = sweep(
	type => 'Step::Temperature',
	instrument => $lakeshore,
	#from => $T_start,
	#to => $T_stop,
	#step => $T_step,
	 #list => [  4.3,4.375,4.425,4.45, 4.50,4.6], 
	 list => [ 3, 5.2], 
	# list => [  1.5, 2.5,3.5,3.75, 4.0,4.2,4.4,4.8,5.0], 
	# 4, 4.1, 4.2, 4.3, 4.4, 4.5, 4.6, #7
	# 4.75, 4.8, 4.85, 
	# 5.05, 5.1, 5.15, 5.2, 5.25, 5.3, 5.4, 5.5, 5.7, 6], #10
	
	# # tolerance_setpoint => 0.01,#0.01,
    # tolerance_std_dev => 0.03, #0.01,
    observation_time => 20,
	# observation_time => 2,
	
	# tolerance_setpoint => 0.0001,#0.01,
    tolerance_std_dev => 0.01, #0.01,
	
	tolerance_setpoint => 0.001,#0.001,#0.0001,#0.01,
    # tolerance_std_dev => 0.0015, #0.002,#0.0025, #0.0015, #0.01,
    # observation_time => 30,#40,
	delay_in_loop => 2*60,
);



#------------------------------save data

my $folder   = datafolder();

my $datafile = sweep_datafile(
	filename => 'R(B)'.$sample,
	columns => ['B' , 'R' , 'V_in' , 'V_sample' , 'I_in', 'Frequency', 'T' , 'x', 'y'] #I_in 
);

$datafile->add_plot(x => 'B', y => 'R', legend => 'T', plot_options => {grid => 1}, refresh_interval=>(30*60) );

my $meas = sub {
	my $sweep = shift;
	my $T_mess = $lakeshore->get_T();
	my $B_field = $ips->get_field();

	my $xy = $li ->get_xy();
	my $x = $xy ->{x};
	my $y = $xy ->{y};
	my $V = sqrt($x**2 + $y**2);
	
	$li->set_auto_sens(
		r => $V,
		min_sens => 2e-6, #1, 2, 5, nV, muV, mV, V - 100nV-1V
    );
      
	
	my $V_sample = $V / $factor; 
	my $R = $V_sample / $I;

	$sweep->log( B => $B_field, R => $R, V_sample => $V_sample, V_in => $ampl, I_in => $I, Frequency => $f, T => $T_mess , x=> $x, y=> $y);

};
#------------------------measurement

$temperaturesweep->start(
    slave		=> $b_sweep,
	measurement => $meas,
    datafile    => $datafile,
    folder      => $folder, #folder		=> 'R_of_B'.$sample   with my $sample = 'OMC210426A_HB3';
);

#------------------------
# $ips->sweep_to_field(target => $B_start, rate => 0.2);
my $finish = strftime " %H-%M" , localtime;
print "FINISHED! $finish \n";
